/*****************************************************
   ,     10  40  .
 8-    ,      .


Chip type               : ATmega16
Program type            : Application
AVR Core Clock frequency: 1.000000 MHz
Memory model            : Small
External RAM size       : 0
Data Stack size         : 256
*****************************************************/

#include <mega16.h>
#include <stdio.h>
#include <delay.h>
#include <io.h>
#asm
   .equ __lcd_port=0x18 ;PORTB
#endasm
#include <lcd.h>

//
void Zad_temp();
void Chek_Botton();
void Check_bounds();
void Check_OK();
void ADC_IN();
void Temp_Ind();
void Out_ON();
void Out_OFF();


// 
volatile short ADC_VALUE;
volatile char D6_OK;
volatile char T_MIN;
volatile char COUNTER;
volatile char D4_UP;
volatile char T_MAX;
volatile char D5_DOWN;
volatile char SENSOR_SEL;
volatile char SENSOR;
volatile char TEMP_def;
volatile char T_ZAD;

//LCD
char lcd_buffer[33];

//  
static void __ADC_SampleADC();
static short __ADC_ReadAsInt();


/*************************************
//ADC interrupt service routine
interrupt [ADC_INT] void adc_isr(void)
{
unsigned int iRetVal;
// Read the AD conversion result
iRetVal=ADCW;
// Place your code here

}
**************************************/

// 
void __ADC_SampleADC()
{
        char dda;
        volatile char cnt;
		
        dda = DDRA;

        //  
        DDRA &= ~(1 << COUNTER);
		
		ADCSRA = 0x8B;   //0b10001011 - ADC Enable; ADC Interrupt Enable; ADC Prescaler-8; 
		#asm("sei")
        ADMUX = 0x20 | COUNTER;  //0b00100xxx - ADC Left Adjust Result

        //  25 
        cnt = 0;
        while (cnt < 0x19) cnt++;

        // 
        ADCSRA |= 0x40;
        while(ADCSRA & 0x40);

        DDRA = dda;
}
static short __ADC_ReadAsInt()
{
        int iRetVal;
        __ADC_SampleADC();
        iRetVal = (ADCL >> 6);
        iRetVal |= (ADCH << 2);
        return (iRetVal);
}

void Zad_temp()
{
    T_MIN = 10;								//  T_min = 10   
    T_MAX = 40;								//  T_max = 40   
    T_ZAD = (T_MAX - T_MIN) / 2 + T_MIN;	//      :
}

void Chek_Botton()
{
    // D4
    DDRD = DDRD & 0x10;
    D4_UP = (PIND.4 == 0);

    // D5
    DDRD = DDRD & 0x20;
    D5_DOWN = (PIND.5 == 0);

    // D6
    DDRD = DDRD & 0x40;
    D6_OK = (PIND == 0);


    //  D4_UP?   
    if (D4_UP == 0)
    {
        // . Delay: 200 ms
        delay_ms(200);
        // "+"?
        if (D4_UP == 0)
        {
         T_ZAD = T_ZAD + 1;
        }
    }

    //  D5_DOWN?   
    if (D5_DOWN == 0)
    {
        // . Delay: 200 ms
        delay_ms(200);
        // "-"?
        if (D5_DOWN == 0)
        {
         T_ZAD = T_ZAD - 1;
        }
    }
}

void Check_bounds()
{
    //     T_zad>T_max?
    if (T_ZAD>T_MAX)
    {
    T_ZAD = T_MAX;
    }

    //     T_zad<T_min?
    if (T_ZAD<T_MIN)
    {
    T_ZAD = T_MIN;
    }
}

void Check_OK()
{
// 
lcd_clear();
B__Check_OK:
    //  "OK" 
     if (D6_OK == 0) 
    {
            // . Delay: 200 ms
            delay_ms(200);
            // D6_OK?
            if (D6_OK == 0)
            {
            //  
            goto A__Check_OK;
            }
    }
else  
    {
    // : Chek_Botton
    Chek_Botton();
    // : Check_bounds
    Check_bounds();    
    };

//    
sprintf(lcd_buffer," Set temp %u grd",T_ZAD);
lcd_init(16);
lcd_gotoxy(0, 0);
lcd_puts(lcd_buffer);

//  
goto B__Check_OK;   

//  
A__Check_OK:

// 
lcd_clear();
}

void ADC_IN()
{
    //   
    while (COUNTER<8)
    {
        // 
        DDRD = DDRD & 0x07;
        SENSOR_SEL = PIND & 0x07;
		
		// : __ADC_ReadAsInt()
        __ADC_ReadAsInt();
        ADC_VALUE = __ADC_ReadAsInt();

        //Temp_Ind
        // : Temp_Ind
        Temp_Ind();

        // Counter
        if (COUNTER==7)
        
        { 
         // 
         COUNTER = 0;
        }
         
        else
   		{
         // 
         COUNTER = COUNTER + 1;
        }
    }
}

void Temp_Ind()
{
    //    
    TEMP_def = ADC_VALUE / 33 + T_MIN;
    
    // 
    if (TEMP_def<T_ZAD)
    {
    // : Out_ON
    Out_ON();
    }

    // 
    if (TEMP_def>T_ZAD)
    {
    // : Out_OFF
    Out_OFF();
    }
   
    //   
    if (COUNTER==SENSOR_SEL)
    {
    SENSOR = SENSOR_SEL + 1;
    delay_ms(50);
	sprintf(lcd_buffer," Temp sensor %u  %u grd",SENSOR,TEMP_def);
    lcd_init(16);
    lcd_gotoxy(1, 0);
    lcd_puts(lcd_buffer);
    delay_ms(200);
    }
}

void Out_ON()
{
//Output_ON
DDRC = DDRC | (1<<COUNTER);
PORTC = PORTC & ~(1 << COUNTER);
}

void Out_OFF()
{
//Output_ON
DDRC = DDRC | (1<<COUNTER);
PORTC = (PORTC & (1 << COUNTER));
}

void main(void)
{
    //
    MCUCSR=0x00;
    #asm("wdr")
    
    // : Zad_temp
    Zad_temp();

    // : Check_OK
    Check_OK();

    //  . Counter = 0
    COUNTER = 0;

    // : ADC_IN
    ADC_IN();
}
